@@ -11,6 +11,9 @@ from pathlib import Path
 from typing import Dict, List, Optional, Union
 
 import tomlkit
+import yaml
+from click import echo
+from colorama import Fore
 
 logger = logging.getLogger(__name__)
 
@@ -28,6 +31,19 @@ def find_pyproject(src: Path) -> Optional[Path]:
     return None
 
 
+def find_djlint_rules(src: Path) -> Optional[Path]:
+    """Search upstream for a pyprojec.toml file."""
+
+    for directory in [src, *src.resolve().parents]:
+
+        candidate = directory / ".djlint_rules.yaml"
+
+        if candidate.is_file():
+            return candidate
+
+    return None
+
+
 def load_pyproject_settings(src: Path) -> Dict:
     """Load djlint config from pyproject.toml."""
 
@@ -44,6 +60,45 @@ def load_pyproject_settings(src: Path) -> Dict:
     return djlint_content
 
 
+def validate_rules(rules: List) -> List:
+    """Validate a list of linter rules. Returns valid rules."""
+    clean_rules = []
+
+    for rule in rules:
+        # check for name
+        warning = 0
+        name = rule["rule"].get("name", "undefined")
+        if "name" not in rule["rule"]:
+            warning += 1
+            echo(Fore.RED + "Warning: A rule is missing a name! 😢")
+        if "patterns" not in rule["rule"]:
+            warning += 1
+            echo(Fore.RED + f"Warning: Rule {name} is missing a pattern! 😢")
+        if "message" not in rule["rule"]:
+            warning += 1
+            echo(Fore.RED + f"Warning: Rule {name} is missing a message! 😢")
+
+        if warning == 0:
+            clean_rules.append(rule)
+
+    return clean_rules
+
+
+def load_custom_rules(src: Path) -> List:
+    """Load djlint config from pyproject.toml."""
+
+    djlint_content: List = []
+    djlint_rules_file = find_djlint_rules(src)
+
+    if djlint_rules_file:
+        djlint_content = yaml.load(
+            Path(djlint_rules_file).read_text(encoding="utf8"),
+            Loader=yaml.SafeLoader,
+        )
+
+    return djlint_content
+
+
 def build_custom_blocks(custom_blocks: Union[str, None]) -> Optional[str]:
     """Build regex string for custom template blocks."""
     if custom_blocks:
@@ -82,6 +137,7 @@ class Config:
             "jinja": ["D", "N", "M"],
             "nunjucks": ["D", "J", "M"],
             "handlebars": ["D", "J", "N"],
+            "golang": ["D", "J", "N", "M"],
         }
 
         self.profile_code: List[str] = profile_dict.get(
@@ -91,6 +147,24 @@ class Config:
             profile or djlint_settings.get("profile", "all")
         ).lower()
 
+        # load linter rules
+        rule_set = validate_rules(
+            yaml.load(
+                (Path(__file__).parent / "rules.yaml").read_text(encoding="utf8"),
+                Loader=yaml.SafeLoader,
+            )
+            + load_custom_rules(Path(src))
+        )
+
+        self.linter_rules = list(
+            filter(
+                lambda x: x["rule"]["name"] not in self.ignore.split(",")
+                and x["rule"]["name"][0] not in self.profile_code
+                and self.profile not in x["rule"].get("exclude", []),
+                rule_set,
+            )
+        )
+
         # base options
         self.indent: str = (indent or int(djlint_settings.get("indent", 4))) * " "
 
@@ -114,8 +188,11 @@ class Config:
             | node_modules
             | __pypackages__
         """
+
         self.exclude: str = djlint_settings.get("exclude", default_exclude)
+
         extend_exclude: str = djlint_settings.get("extend_exclude", "")
+
         if extend_exclude:
             self.exclude += r" | " + r" | ".join(
                 re.escape(x.strip()) for x in extend_exclude.split(",")
@@ -126,12 +203,18 @@ class Config:
             "blank_line_after_tag", None
         )
 
-        # contents of tags will not be formatted, but tags will be formatted
+        # contents of tags will not be formatted
         self.ignored_block_opening: str = r"""
               <style
             | {\*
             | <\?php
             | <script
+            | <!--
+            | [^\{]{\#
+            | <pre
+            | <textarea
+            | {%[ ]djlint:off[ ]%}
+            | {%[ ]+?comment[ ]+?[^(?:%})]*?%}
         """
 
         self.ignored_block_closing: str = r"""
@@ -139,322 +222,16 @@ class Config:
             | \*}
             | \?>
             | </script
-        """
-
-        # contents of tags will not be formated and tags will not be formatted
-        self.ignored_group_opening: str = r"""
-              <!--
-            | [^\{]{\#
-            | <pre
-            | <textarea
-        """
-
-        self.ignored_group_closing: str = r"""
-              -->
+            |  -->
             | \#}
             | </pre
             | </textarea
+            | {%[ ]djlint:on[ ]%}
+            | {%[ ]+?endcomment[ ]+?%}
         """
 
-        # the contents of these tag blocks will be indented, then unindented
-        self.tag_indent: str = (
-            r"""
-              (?:\{\{\#|\{%-?)[ ]*?
-                (
-                      if
-                    | for
-                    | block
-                    | else
-                    | spaceless
-                    | compress
-                    | addto
-                    | language
-                    | with
-                    | assets
-                    | verbatim
-                    | autoescape
-                    | comment
-                    | filter
-                    | each
-                    """
-            + self.custom_blocks
-            + r"""
-                )
-            | (?:<
-                (?:
-                      html
-                    | head
-                    | body
-                    | div
-                    | a
-                    | nav
-                    | ul
-                    | ol
-                    | dl
-                    | dd
-                    | dt
-                    | li
-                    | table
-                    | thead
-                    | tbody
-                    | tr
-                    | th
-                    | td
-                    | blockquote
-                    | select
-                    | form
-                    | option
-                    | cache
-                    | optgroup
-                    | fieldset
-                    | legend
-                    | label
-                    | header
-                    | main
-                    | section
-                    | aside
-                    | footer
-                    | figure
-                    | figcaption
-                    | video
-                    | span
-                    | p
-                    | g
-                    | svg
-                    | h\d
-                    | button
-                    | img
-                    | path
-                    | script
-                    | style
-                    | details
-                    | summary
-                )
-              )
-        """
-        )
-
-        self.tag_unindent: str = r"""^
-              (?:
-                  (?:\{\{\/)
-                | (?:\{%-?[ ]*?end)
-              )
-            | (?:</
-                (?:
-                      html
-                    | head
-                    | body
-                    | div
-                    | a
-                    | nav
-                    | ul
-                    | ol
-                    | dl
-                    | dd
-                    | dt
-                    | li
-                    | table
-                    | thead
-                    | tbody
-                    | tr
-                    | th
-                    | td
-                    | blockquote
-                    | select
-                    | form
-                    | option
-                    | optgroup
-                    | fieldset
-                    | legend
-                    | label
-                    | header
-                    | cache
-                    | main
-                    | section
-                    | aside
-                    | footer
-                    | figure
-                    | figcaption
-                    | video
-                    | span
-                    | p
-                    | g
-                    | svg
-                    | h\d
-                    | button
-                    | img
-                    | path
-                    | script
-                    | style
-                    | details
-                    | summary
-                )
-              )
-        """
-
-        # these tags should be unindented and next line will be indented
-        self.tag_unindent_line: str = r"""
-              (?:\{%-?[ ]*?(?:elif|else|empty))
-            | (?:
-                \{\{[ ]*?
-                (
-                    (?:else|\^)
-                    [ ]*?\}\}
-                )
-              )
-        """
-
-        # reduce empty lines greater than  x to 1 line
-        self.reduce_extralines_gt = 2
-
-        # if lines are longer than x
-        self.max_line_length = 120
-        self.format_long_attributes = True
-
-        # pattern used to find attributes in a tag
-        # order is important.
-        # 1. attributes="{% if %}with if or for statement{% endif %}"
-        # 2. attributes="{{ stuff in here }}"
-        # 3. {% if %}with if or for statement{% endif %}
-        # 4. attributes="normal html"
-        # 5. require | checked | otherword | other-word
-        # 6. {{ stuff }}
-        template_if_for_pattern = (
-            r"(?:{%-?\s?(?:if|for)[^}]*?%}(?:.*?{%\s?end(?:if|for)[^}]*?-?%})+?)"
-        )
-        self.attribute_pattern: str = (
-            r"""
-            (?:[^\s]+?=(?:\"[^\"]*?"""
-            + template_if_for_pattern
-            + r"""[^\"]*?\"|\'[^\']*?"""
-            + template_if_for_pattern
-            + r"""[^\']*?\'))
-            | (?:[^\s]+?=(?:\"[^\"]*?{{.*?}}[^\"]*?\"|\'[^\']*?{{.*?}}[^\']*?\'))
-            | """
-            + template_if_for_pattern
-            + r"""
-            | (?:[^\s]+?=(?:\".*?\"|\'.*?\'))
-            | required
-            | checked
-            | [\w|-]+
-            | [\w|-]+=[\w|-]+
-            | {{.*?}}
-        """
-        )
-
-        self.tag_pattern: str = r"""
-            (<\w+?[^>]*?)((?:\n[^>]+?)+?)(/?\>)
-        """
-
-        self.start_template_tags: str = (
-            r"""
-              if
-            | for
-            | block
-            | spaceless
-            | compress
-            | load
-            | assets
-            | addto
-            | language
-            | with
-            | assets
-            | autoescape
-            | comment
-            | filter
-            | verbatim
-            | each
-            """
-            + self.custom_blocks
-            + r"""
-        """
-        )
-
-        self.break_template_tags: str = (
-            r"""
-              if
-            | end
-            | for
-            | block
-            | endblock
-            | else
-            | spaceless
-            | compress
-            | load
-            | include
-            | assets
-            | addto
-            | language
-            | with
-            | assets
-            | autoescape
-            | comment
-            | filter
-            | elif
-            | resetcycle
-            | verbatim
-            | each
-            """
-            + self.custom_blocks
-            + r"""
-        """
-        )
-
-        self.ignored_blocks: str = r"""
-              <(script|style|pre|textarea).*?</(\1)>
-            | <!--.*?-->
-            | {\*.*?\*}
-            | {\#.*?\#}
-            | <\?php.*?\?>
-        """
-
-        self.ignored_inline_blocks: str = r"""
-              <!--.*?-->
-            | {\*.*?\*}
-            | {\#.*?\#}
-            | <\?php.*?\?>
-        """
-
-        self.single_line_html_tags: str = r"""
-              button
-            | a
-            | h1
-            | h2
-            | h3
-            | h4
-            | h5
-            | h6
-            | td
-            | th
-            | strong
-            | small
-            | em
-            | icon
-            | span
-            | title
-            | link
-            | path
-            | label
-            | div
-            | li
-        """
-
-        self.always_single_line_html_tags: str = r"""
-              link
-            | img
-            | meta
-            | source
-        """
-
-        self.single_line_template_tags: str = r"""
-              if
-            | for
-            | block
-            | with
-            | comment
-        """
-
-        self.break_html_tags: str = r"""
+        # all html tags possible
+        self.indent_html_tags: str = r"""
               a
             | abbr
             | acronym
@@ -578,3 +355,275 @@ class Config:
             | video
             | wbr
         """
+
+        self.indent_template_tags: str = (
+            r"""  if
+                | for
+                | block
+                | else
+                | spaceless
+                | compress
+                | addto
+                | language
+                | with
+                | assets
+                | verbatim
+                | autoescape
+                | filter
+                | each
+            """
+            + self.custom_blocks
+        )
+
+        # the contents of these tag blocks will be indented, then unindented
+        self.tag_indent: str = (
+            r"""
+              (?:\{\{\#|\{%-?)[ ]*?
+                ("""
+            + self.indent_template_tags
+            + r"""
+                )
+            | (?:<
+                (?:
+                    """
+            + self.indent_html_tags
+            + """
+                )
+              )
+        """
+        )
+
+        self.tag_unindent: str = (
+            r"""^
+              (?:
+                  (?:\{\{\/)
+                | (?:\{%-?[ ]*?end)
+              )
+            | (?:</
+                (?:
+                    """
+            + self.indent_html_tags
+            + """
+                )
+              )
+        """
+        )
+
+        # these tags should be unindented and next line will be indented
+        self.tag_unindent_line: str = r"""
+              (?:\{%-?[ ]*?(?:elif|else|empty))
+            | (?:
+                \{\{[ ]*?
+                (
+                    (?:else|\^)
+                    [ ]*?\}\}
+                )
+              )
+        """
+
+        self.break_before = r"(?<!\n[ ]*?)"
+
+        # if lines are longer than x
+        self.max_line_length = 120
+
+        self.format_long_attributes = True
+
+        # pattern used to find attributes in a tag
+        # order is important.
+        # 1. attributes="{% if %}with if or for statement{% endif %}"
+        # 2. attributes="{{ stuff in here }}"
+        # 3. {% if %}with if or for statement{% endif %}
+        # 4. attributes="normal html"
+        # 5. require | checked | otherword | other-word
+        # 6. {{ stuff }}
+
+        self.template_if_for_pattern = (
+            r"(?:{%-?\s?(?:if|for)[^}]*?%}(?:.*?{%\s?end(?:if|for)[^}]*?-?%})+?)"
+        )
+        self.attribute_pattern: str = (
+            r"""
+            (?:[^\s]+?=(?:\"[^\"]*?"""
+            + self.template_if_for_pattern
+            + r"""[^\"]*?\"|\'[^\']*?"""
+            + self.template_if_for_pattern
+            + r"""[^\']*?\'))
+            | (?:[^\s]+?=(?:\"[^\"]*?{{.*?}}[^\"]*?\"|\'[^\']*?{{.*?}}[^\']*?\'))
+            | """
+            + self.template_if_for_pattern
+            + r"""
+            | (?:[^\s]+?=(?:\"(?:[^\"]*?{%[^}]*?%}[^\"]*?)+?\"))
+            | (?:[^\s]+?=(?:\'(?:[^\']*?{%[^}]*?%}[^\']*?)+?\'))
+            | (?:[^\s]+?=(?:\".*?\"|\'.*?\'))
+            | required
+            | checked
+            | [\w|-]+
+            | [\w|-]+=[\w|-]+
+            | {{.*?}}
+        """
+        )
+
+        self.attribute_style_pattern: str = r"(.*?)(style=)([\"|'])(([^\"']+?;)+?)\3"
+
+        self.start_template_tags: str = (
+            r"""
+              if
+            | for
+            | block
+            | spaceless
+            | compress
+            | load
+            | assets
+            | addto
+            | language
+            | with
+            | assets
+            | autoescape
+            | filter
+            | verbatim
+            | each
+            """
+            + self.custom_blocks
+            + r"""
+        """
+        )
+
+        self.break_template_tags: str = (
+            r"""
+              if
+            | end
+            | for
+            | block
+            | endblock
+            | else
+            | spaceless
+            | compress
+            | load
+            | include
+            | assets
+            | addto
+            | language
+            | with
+            | assets
+            | autoescape
+            | filter
+            | elif
+            | resetcycle
+            | verbatim
+            | each
+            """
+            + self.custom_blocks
+            + r"""
+        """
+        )
+
+        self.ignored_blocks: str = r"""
+              <(script|style|pre|textarea).*?</(\1)>
+            | {%[ ]djlint:off[ ]%}.*?{%[ ]djlint:on[ ]%}
+            | <!--.*?-->
+            | {\*.*?\*}
+            | {\#.*?\#}
+            | <\?php.*?\?>
+            | {\%[ ]trans[ ][^}]*?\%}
+            | {%[ ]+?comment[ ]+?[^(?:%})]*?%}.*?{%[ ]+?endcomment[ ]+?%}
+        """
+
+        self.ignored_inline_blocks: str = r"""
+              <!--.*?-->
+            | {\*.*?\*}
+            | {\#.*?\#}
+            | <\?php.*?\?>
+            | {%[ ]+?comment[ ]+?[^(?:%})]*?%}.*?{%[ ]+?endcomment[ ]+?%}
+        """
+
+        self.optional_single_line_html_tags: str = r"""
+              button
+            | a
+            | h1
+            | h2
+            | h3
+            | h4
+            | h5
+            | h6
+            | td
+            | th
+            | strong
+            | small
+            | em
+            | icon
+            | span
+            | title
+            | link
+            | path
+            | label
+            | div
+            | li
+            | script
+            | style
+        """
+
+        self.always_self_closing_html_tags: str = r"""
+              link
+            | img
+            | meta
+            | source
+            | br
+            | input
+        """
+
+        self.optional_single_line_template_tags: str = r"""
+              if
+            | for
+            | block
+            | with
+        """
+
+        self.break_html_tags: str = r"""
+              html
+            | head
+            | body
+            | div
+            | a
+            | nav
+            | ul
+            | ol
+            | dl
+            | dd
+            | dt
+            | li
+            | table
+            | thead
+            | tbody
+            | tr
+            | th
+            | td
+            | blockquote
+            | select
+            | form
+            | option
+            | optgroup
+            | fieldset
+            | legend
+            | label
+            | header
+            | cache
+            | main
+            | section
+            | aside
+            | footer
+            | figure
+            | figcaption
+            | video
+            | span
+            | p
+            | g
+            | svg
+            | h\d
+            | button
+            | img
+            | path
+            | picture
+            | script
+            | style
+            | details
+            | summary
+        """
